#include <Servo.h>

// pins
const int trigPin = 2; // trig pin for ultrasonic sensor
const int echoPin = 3; // echo pin for ultrasonic sensor
const int servoPin = 4; // pin for writing to servo

// distance
float minCheckDistance = 2.0; // min check distance in CM
float maxCheckDistance = 8.0; // max check distance in CM
float echoDuration, distance;
float averageDistance[10] = {0};

// servo
Servo serv;
const int minPos = 75; // servo maximum position while dipping
const int maxPos = 100; // servo minimum position while dipping
const int restingPos = 130; // servo resting position when not dipping
int targetPos = restingPos;
int pos = restingPos;

// time
unsigned long time = millis();
unsigned long startTime = time;
unsigned long dipDuration = 60000; // duration of comenced dipping in miliseconds
bool dipping = false;
bool reset = false;

void setup() {
  // enbable pins
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);
  serv.attach(servoPin);
  serv.write(pos);
  delay(300);
}

void loop() {
  time = millis();

  // insert a distance into the array
  distance = checkDistance();
  for(int i=0; i < 9; i++){
    averageDistance[i] = averageDistance[i+1];
  }
  averageDistance[9] = distance;

  // handle the setting down cup and removing it again
  if(checkAverageDistance()){
    if(time >= startTime+dipDuration && dipping){
      dipping = false;
      reset = true;
    }else if(!dipping && !reset){
      startTime = time;
      dipping = true;
    }
  }else if(!checkAverageDistance()){
    dipping = false;
    reset = false;
  }

  // handle servo
  if(!dipping){
    targetPos = restingPos;
  }else{
    if (pos >= maxPos){
      targetPos = minPos;
    }else if(pos <= minPos){
      targetPos = maxPos;
    }
  }

  moveServo();

  delay(50);
}

void moveServo(){
    if (pos < targetPos){
      pos += 1;
      serv.write(pos);
    }else if (pos > targetPos){
      pos -= 1;
      serv.write(pos);
    }
}

float checkAverageDistance(){
  // check if the average distance in the last 10 echoes is still around our target (3 echoes or more)
  int avg = 0;
  for (byte i=0; i < 10; i++){
    if (averageDistance[i] <= maxCheckDistance && averageDistance[i] >= minCheckDistance){
      avg++;
    }
  }
  return avg >= 3;
}

float checkDistance(){
  // pulse the signal so we get echoes
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);

  echoDuration = pulseIn(echoPin, HIGH);
  return (echoDuration * 0.0343) / 2;
}

